#!/bin/bash

PATH=$PATH:/var/epages

#=======================================================================
# add-cartridges-to-bu.sh
#=======================================================================

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#-----------------------------------------------------------------------
# define variables and functions
#-----------------------------------------------------------------------

BN=${0##*/}
DN=$(cd $(dirname $0) && pwd)

usage() {
    [[ -n $1 ]] && echo -e "Error:\n  $1\n"
  cat << USAGE_END
Usage: $BN -s STORE -a ADDITIONAL_CARTRIDGES [-x CARTRIDGES_XML] [-n]

Description:
  Adds ADDITIONAL_CARTRIDGES to STORE.
  With CARTRIDGES_XML, uses this Cartridges.xml instead of
  $EPAGES_CARTRIDGES/Cartridges.xml
  With -n do not synchronize installed cartridges.

Example:
  $BN -s Store -a 'XX::YY XX::ZZ'
USAGE_END
  exit 1
}

. /etc/default/epages6

unset STORE ADDITIONAL_CARTRIDGES CARTRIDGES_XML SKIP_SYNCHRONIZE

while getopts s:a:x:n option ; do
  case $option in
    s) STORE=$OPTARG ;;
    a) ADDITIONAL_CARTRIDGES=$OPTARG ;;
    x) CARTRIDGES_XML=$(readlink -m "$OPTARG") ;;
    n) SKIP_SYNCHRONIZE=1 ;;
    *) usage ;;
  esac
done

[[ -n $STORE ]] || usage "Option -s missing"
[[ -n $ADDITIONAL_CARTRIDGES ]] || usage "Options -a missing"

CARTRIDGES_CONF=$EPAGES_CONFIG/Cartridges.d/$STORE.conf
if [[ ! -f $CARTRIDGES_CONF ]] ; then
  echo "$BN: no $CARTRIDGES_CONF found, exiting."
  exit 1
fi

DATABASE_CONF=$EPAGES_CONFIG/Database.d/$STORE.conf
if [[ ! -f $DATABASE_CONF ]] ; then
  echo "$BN: no $DATABASE_CONF found, exiting."
  exit 1
fi

STORETYPE=$(ini-config.sh -f $DATABASE_CONF -s Connection -k StoreType -G)
if [[ -z $STORETYPE ]] ; then
  echo "$BN: no StoreType in $DATABASE_CONF found, exiting."
  exit 1
fi

set -e

#-----------------------------------------------------------------------
# install ADDITIONAL_CARTRIDGES
#-----------------------------------------------------------------------

unset NEW_CARTRIDGES
for CARTRIDGE in $ADDITIONAL_CARTRIDGES ; do

  # go on if cartridge is already installed
  INSTALLED_VERSION=$(ini-config.sh -f $CARTRIDGES_CONF -s Cartridges -k "$CARTRIDGE" -G)
  if [[ -n $INSTALLED_VERSION ]] ; then
    echo "$BN: skip $CARTRIDGE: cartridge already installed"
    continue
  fi

  # go on if cartridge directory is missing
  if ! pushd "$EPAGES_CARTRIDGES/${CARTRIDGE//:://}" >/dev/null ; then
    echo "$BN: skip $CARTRIDGE: cartridge directory missing"
    continue
  fi

  # go on if cartridge Makefile.PL is missing
  if ! ls "$EPAGES_CARTRIDGES/${CARTRIDGE//:://}"/Makefile.PL >/dev/null ; then
    echo "$BN: skip $CARTRIDGE: no Makefile.PL for cartridge"
    popd >/dev/null
    continue
  fi

  # make install register STORE=$STORE
  ECHO $LINENO "$PERL $(pwd)/Makefile.PL"
  $PERL Makefile.PL
  ECHO $LINENO "make install register STORE=$STORE"
  make install register STORE=$STORE
  NEW_CARTRIDGES="$NEW_CARTRIDGES $CARTRIDGE"

  popd >/dev/null
done

# exit here unless new cartridges were installed
[[ -n $NEW_CARTRIDGES ]] || exit 0

#-----------------------------------------------------------------------
# add NEW_CARTRIDGES to Cartridges.xml
#-----------------------------------------------------------------------

if [[ -n $CARTRIDGES_XML ]] ; then
  ECHO $LINENO "$DN/update_cartridges_xml.sh -s $STORETYPE -a '$NEW_CARTRIDGES' -x $CARTRIDGES_XML"
  $DN/update_cartridges_xml.sh -s $STORETYPE -a "$NEW_CARTRIDGES" -x $CARTRIDGES_XML
else
  ECHO $LINENO "$DN/update_cartridges_xml.sh -s $STORETYPE -a '$NEW_CARTRIDGES'"
  $DN/update_cartridges_xml.sh -s $STORETYPE -a "$NEW_CARTRIDGES"
fi

#-----------------------------------------------------------------------
# check whether to update WebRoot/StoreTypes/VERSION
#-----------------------------------------------------------------------

cd $EPAGES_CARTRIDGES

unset UPDATE_STORETYPES_DIR

if [[ $NEW_CARTRIDGES =~ DE_EPAGES::Dojo || $NEW_CARTRIDGES =~ DE_EPAGES::JQuery ]] ; then
  UPDATE_STORETYPES_DIR=1
else 
  # allow ** for matching directory paths
  shopt -s globstar
  for CARTRIDGE in $NEW_CARTRIDGES ; do
    echo $CARTRIDGE
    if ls -d ${CARTRIDGE//:://}/{Templates/Dictionary.*.xml,Data/javascript} \
      2>/dev/null | grep -q . ; then
      UPDATE_STORETYPES_DIR=1
      break
    fi
    if ls ${CARTRIDGE//:://}/Data/Public/{GUI,lib}/{**/,}*.{css,js,tmpl.html,xml} \
      2>/dev/null | grep -q . ; then
      UPDATE_STORETYPES_DIR=1
      break
    fi
  done
  # disable ** for matching directory paths
  shopt -u globstar
fi

#-----------------------------------------------------------------------
# update WebRoot/StoreTypes/VERSION
#-----------------------------------------------------------------------

cd $EPAGES_CARTRIDGES/DE_EPAGES

if [[ -n $UPDATE_STORETYPES_DIR ]] ; then

  $PERL Makefile.PL
  VERSION=$(ini-config.sh -f makefile -k EPAGES_VERSION_NUMBER -G)

  for TYPE in Public javascript ; do
    ECHO $LINENO "$PERL Installer/Scripts/copyCartridgeData.pl ALL -type $TYPE -version $VERSION -storename $STORE"
    $PERL Installer/Scripts/copyCartridgeData.pl ALL -type $TYPE -version $VERSION -storename $STORE
  done

  ECHO $LINENO "make install_storetype STORE_TYPE=$STORETYPE"
  make install_storetype STORE_TYPE=$STORETYPE
 
  for STORE in $STORES ; do
    ECHO $LINENO "$EPAGES_PERL/bin/nodejs.sh $EPAGES_PERL/nodejs.d/lib/node_modules/node-epages6/bin/epnode6.js build-ui -s $STORE -v $VERSION"
    $EPAGES_PERL/bin/nodejs.sh $EPAGES_PERL/nodejs.d/lib/node_modules/node-epages6/bin/epnode6.js build-ui -s $STORE -v $VERSION
  done
fi

#-----------------------------------------------------------------------
# synchronize installed cartridges
#-----------------------------------------------------------------------

if [[ -z $SKIP_SYNCHRONIZE ]] ; then
  ECHO $LINENO "$PERL ShopConfiguration/Scripts/synccartridges.pl -storename $STORE"
  $PERL ShopConfiguration/Scripts/synccartridges.pl -storename $STORE
fi

exit 0
