#!/bin/bash

DN=$(cd $(dirname $0) && pwd)

#==============================================================================
# update-epages.sh
#==============================================================================

if [ -z "$BASH_VERSION" ] ; then
  echo "$0: this script can only be executed by bash"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 {Options}"
  echo "Options:"
  echo "  --jwtsecret SECRET : set jwtsecret in each Database.d/STORE.conf"
  echo "  --unity-host HOST : unity host name"
  echo "  --unity-port PORT : unity port number"
  echo "  --unity-http PORT : unity web server port"
  echo "  --unity-https PORT : unity SSL web server port"
  echo "  --subdomain-routing {1|0} : unity uses subdomain routing (or not)"
  echo "  --where {db|conf}: update only database or configuration file (default: both)"
  echo "Examples:"
  echo "  $0 --subdomain-routing 0 --unity-port 7000 --unity-host localhost --jwtsecret xy"
  echo "  $0 --unity-port 7000"
  exit 1
}

TEMP=$(getopt -o '' --long help,where:,jwtsecret:,unity-host:,unity-port:,unity-http:,unity-https:,subdomain-routing: -n $0 -- "$@") || usage
eval set -- "$TEMP"

unset JWTSECRET UNITY_HOST UNITY_PORT UNITY_HTTP_PORT UNITY_HTTPS_PORT SUBDOMAIN_ROUTING WHERE
while true ; do
  case "$1" in
    --help) usage ;;
    --where) [[ -z $2 ]] && usage ; WHERE=$2 ; shift 2 ;;
    --jwtsecret) [[ -z $2 ]] && usage ; JWTSECRET=$2 ; shift 2 ;;
    --unity-host) [[ -z $2 ]] && usage ; UNITY_HOST=$2 ; shift 2 ;;
    --unity-port) [[ -z $2 ]] && usage ; UNITY_PORT=$2 ; shift 2 ;;
    --unity-http) [[ -z $2 ]] && usage ; UNITY_HTTP_PORT=$2 ; shift 2 ;;
    --unity-https) [[ -z $2 ]] && usage ; UNITY_HTTPS_PORT=$2 ; shift 2 ;;
    --subdomain-routing) [[ -z $2 ]] && usage ; SUBDOMAIN_ROUTING=$2 ; shift 2 ;;
    --) shift ; break ;;
    *) echo $@ ; usage ;;
  esac
done

# no other arguments allowed
[[ -n $1 ]] && usage

JWTSECRET=${JWTSECRET//\"/}
if [[ $WHERE != db && -z "$JWTSECRET$UNITY_HOST$UNITY_PORT$UNITY_HTTP_PORT$UNITY_HTTPS_PORT$SUBDOMAIN_ROUTING" ]] ; then
  usage "$0: script requires at least 1 option"
fi

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

PATH=$PATH:/var/epages

. /etc/default/epages6
export mktempdir

touch $EPAGES_CONFIG/Database.d || { echo "$0: failed"; exit 1; }
touch $EPAGES_CONFIG/epages.conf || { echo "$0: failed"; exit 1; }

#-----------------------------------------------------------------------
# printCartridgesXml.pl
#-----------------------------------------------------------------------

CARTRIDGES_TXT=$(mktempdir epages-unity)/cartridges.txt
ECHO $LINENO "$PERL $DN/printCartridgesXml.pl > $CARTRIDGES_TXT"
$PERL $DN/printCartridgesXml.pl > $CARTRIDGES_TXT

#-----------------------------------------------------------------------
# update /etc/default/epages6 (unless --where db)
#-----------------------------------------------------------------------

if [[ $WHERE != db && -e /etc/default/epages6 ]] ; then
  FOUND=$(ini-config.sh -f /etc/default/epages6 -k EPAGES_UNITY_HOST -G)
  if [[ ${FOUND:-localhost} != ${UNITY_HOST:-localhost} ]] ; then
    ECHO $LINENO "/etc/default/epages6: EPAGES_UNITY_HOST=$UNITY_HOST"
    ini-config.sh -f /etc/default/epages6 -k EPAGES_UNITY_HOST -v "$UNITY_HOST" -i
  fi
  FOUND=$(ini-config.sh -f /etc/default/epages6 -k EPAGES_UNITY_PORT -G)
  if [[ $FOUND != ${UNITY_PORT:-7000} ]] ; then
    ECHO $LINENO "/etc/default/epages6: EPAGES_UNITY_PORT=${UNITY_PORT:-7000}"
    ini-config.sh -f /etc/default/epages6 -k EPAGES_UNITY_PORT -v "${UNITY_PORT:-7000}" -i
  fi
  if [[ -n $SUBDOMAIN_ROUTING ]] ; then
    FOUND=$(ini-config.sh -f /etc/default/epages6 -k HTTPD_OPTS -G)
    if ! [[ $HTTPD_OPTS =~ DUNITY_$SUBDOMAIN_ROUTING ]] ; then
      val=${HTTPD_OPTS//-DUNITY_URL/}
      val=$(echo "${val//-DUNITY_VHOST/} -DUNITY_$SUBDOMAIN_ROUTING")
      ECHO $LINENO "/etc/default/epages6: HTTPD_OPTS=\"$val\""
      ini-config.sh -f /etc/default/epages6 -k HTTPD_OPTS -v "\"$val\"" -i
    fi
  fi
fi

#-----------------------------------------------------------------------
# update epages.conf (unless --where db)
#-----------------------------------------------------------------------

if [[ $WHERE != db ]] ; then
  FOUND=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityUrlScheme -G)
  if [[ -z "$FOUND$STORE" || $FOUND != $UNITY_HOST ]]; then
    ECHO $LINENO "$EPAGES_CONFIG/epages.conf: UnityUrlScheme=${UNITY_HOST:-localhost}"
    ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityUrlScheme -v ${UNITY_HOST:-localhost} -i
  fi

  FOUND=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityApiUrlScheme -G)
  if [[ -z "$FOUND$STORE" || $FOUND != $UNITY_HOST/api/v2 ]]; then
    ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityApiUrlScheme -v ${UNITY_HOST:-localhost}/api/v2 -i
  fi

  if [[ -n $SUBDOMAIN_ROUTING ]]; then
    ECHO $LINENO "$EPAGES_CONFIG/epages.conf: UnityUseSubdomainRouting=$SUBDOMAIN_ROUTING"
    ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityUseSubdomainRouting -v $SUBDOMAIN_ROUTING -i
  fi

  FOUND=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityWebServerPort -G)
  if [[ -z "$FOUND$STORE" || $FOUND != $UNITY_HTTP_PORT ]]; then
    ECHO $LINENO "$EPAGES_CONFIG/epages.conf: UnityWebServerPort=${UNITY_HTTP_PORT:-80}"
    ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityWebServerPort -v ${UNITY_HTTP_PORT:-80} -i
  fi

  FOUND=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnitySSLWebServerPort -G)
  if [[ -z "$FOUND$STORE" || $FOUND != $UNITY_HTTPS_PORT ]]; then
    ECHO $LINENO "$EPAGES_CONFIG/epages.conf: UnitySSLWebServerPort=${UNITY_HTTPS_PORT:-443}"
    ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnitySSLWebServerPort -v ${UNITY_HTTPS_PORT:-443} -i
  fi
fi

#-----------------------------------------------------------------------
# get information from $EPAGES_CONFIG/epages.conf (if --where db)
#-----------------------------------------------------------------------

if [[ $WHERE = db ]] ; then
  UNITY_HOST=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityUrlScheme -G)
  UNITY_HTTP_PORT=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityWebServerPort -G)
  UNITY_HTTPS_PORT=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnitySSLWebServerPort -G)
  SUBDOMAIN_ROUTING=$(ini-config.sh -f $EPAGES_CONFIG/epages.conf -s DE_EPAGES::Object -k UnityUseSubdomainRouting -G)
fi

#-----------------------------------------------------------------------
# update Database.d/STORE.conf
#-----------------------------------------------------------------------

grep '\bDE_EPAGES::Shop\b' $CARTRIDGES_TXT | while read store cartridges ; do
  [[ -f $EPAGES_CONFIG/Database.d/$store.conf ]] || continue
  if [[ $WHERE != db ]] ; then
    FOUND=$(ini-config.sh -f $EPAGES_CONFIG/Database.d/$store.conf -s Connection -k jwtsecret -G)
    [[ -z $FOUND || $FOUND != $JWTSECRET ]] || continue
    ECHO $LINENO "$EPAGES_CONFIG/Database.d/$store.conf: jwtsecret=${JWTSECRET:-this-is-super-secret---not}"
    ini-config.sh -f $EPAGES_CONFIG/Database.d/$store.conf -s Connection -k jwtsecret -v "${JWTSECRET:-this-is-super-secret---not}" -i
  fi

  [[ $WHERE = conf ]] && continue
  SET_PL=/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Scripts/set.pl
  ECHO $LINENO "$PERL $SET_PL -storename Site -path /Stores/$store UnityUrlScheme=${UNITY_HOST:-localhost}"
  $PERL $SET_PL -storename Site -path /Stores/$store UnityUrlScheme=${UNITY_HOST:-localhost}
  $PERL $SET_PL -storename Site -path /Stores/$store UnityApiUrlScheme=${UNITY_HOST:-localhost}/api/v2
  ECHO $LINENO "$PERL $SET_PL -storename Site -path /Stores/$store UnityWebServerPort=${UNITY_HTTP_PORT:-80}"
  $PERL $SET_PL -storename Site -path /Stores/$store UnityWebServerPort=${UNITY_HTTP_PORT:-80}
  ECHO $LINENO "$PERL $SET_PL -storename Site -path /Stores/$store UnitySSLWebServerPort=${UNITY_HTTPS_PORT:-443}"
  $PERL $SET_PL -storename Site -path /Stores/$store UnitySSLWebServerPort=${UNITY_HTTPS_PORT:-443}
  ECHO $LINENO "$PERL $SET_PL -storename Site -path /Stores/$store UnityUseSubdomainRouting=$SUBDOMAIN_ROUTING"
  $PERL $SET_PL -storename Site -path /Stores/$store UnityUseSubdomainRouting=$SUBDOMAIN_ROUTING
done

#-----------------------------------------------------------------------
# exit
#-----------------------------------------------------------------------

exit 0
