#!/bin/bash

PKG=${0##*/} ; PKG=${PKG%.sh} ; PKG=${PKG/spec/epages}

#==============================================================================
# epages-unity.spec
#==============================================================================

export TERM=

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

. $(dirname $0)/fun-epages.sh
export update_epages_environment epages_service
export etc_default enable_ipv6 set_local_ips

. $(dirname $0)/fun-pkg.sh
export pkg_instance touch_pkg_result grep_server

. $(dirname $0)/fun-java.sh
export test_version

. $(dirname $0)/fun-arch.sh
export update_grub_conf

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

# INSTANCE is always:
# last:   0 - remove last version of package
# first:  1 - install the first time
# update: 2 (or higher) - update (depending on the number of versions installed)

#..............................................................................
# epages-unity-pre
#..............................................................................
epages-unity-pre() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-unity-pre INSTANCE RPM_VERSION'
    echo 'description: pre-installation tasks of epages-unity package'
    echo 'example: epages-unity-pre 0 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2
  local UNAME_R=$(uname -r)

  if test_version $UNAME_R -lt 3.10 ; then
    echo "$UNAME_R: docker requires kernel version >= 3.10" 1>&2
    return $(touch_pkg_result $PKG pre 1 "$UNAME_R: docker requires kernel version >= 3.10")
  fi

  [[ $INSTANCE = first ]] || return $(touch_pkg_result $PKG pre 0)

  local MISSING_CREDENTIALS=
  if [[ -z $UNITY_DOCKER_USER && -z $UNITY_DOCKER_PASSWORD ]] ; then
    MISSING_CREDENTIALS='UNITY_DOCKER_USER/UNITY_DOCKER_PASSWORD'
  elif [[ -z $UNITY_DOCKER_USER ]] ; then
    MISSING_CREDENTIALS='UNITY_DOCKER_USER'
  elif [[ -z $UNITY_DOCKER_PASSWORD ]] ; then
    MISSING_CREDENTIALS='UNITY_DOCKER_PASSWORD'
  fi
  if [[ -n $MISSING_CREDENTIALS ]] ; then
    echo "Docker credentials missing: $MISSING_CREDENTIALS" 1>&2
    return $(touch_pkg_result $PKG pre 1 "Docker credentials missing: $MISSING_CREDENTIALS")
  fi

  return $(touch_pkg_result $PKG pre 0)
}

#..............................................................................
# epages-unity-post
#..............................................................................
epages-unity-post() {
  if [[ $# -ne 2 || -z $1 || -z $2 ]] ; then
    echo 'syntax: epages-unity-post INSTANCE RPM_VERSION'
    echo 'description: post-installation tasks of epages-unity package'
    echo 'example: epages-unity-post 1 6.16.4'
    return 1
  fi
  local INSTANCE=$(pkg_instance $1)
  local VERSION=$2
  local RESULT=0
  local MSG=
  local YAML=/srv/epages/unity/conf/docker-compose.yml

  if [[ $INSTANCE = first ]] ; then

    update_grub_conf cgroup_enable=memory swapaccount=1

    [[ -f $YAML ]] || cp -p $YAML.sample $YAML || exit 1

    AUTH_JWT_SECRET=${AUTH_JWT_SECRET//\"/}
    [[ -z $AUTH_JWT_SECRET ]] || sed -i "s,this-is-super-secret---not,$AUTH_JWT_SECRET," $YAML
    [[ -z $EPAGES_UNITY_PORT ]] || sed -i "s,7000:,$EPAGES_UNITY_PORT:," $YAML

    if systemctl -q enable docker ; then
      if systemctl -q start docker ; then
	if pip install backports.ssl_match_hostname -U ; then
	  if ! pip install docker-compose ; then
	    RESULT=1; MSG='Command failed: pip install docker-compose'
	  fi
	else
	  RESULT=1; MSG='Command failed: pip install backports.ssl_match_hostname -U'
	fi
      else
	RESULT=1; MSG='Command failed: systemctl -q start docker'
      fi
    else
      RESULT=1; MSG='Command failed: systemctl -q enable docker'
    fi
  fi

  [[ $RESULT = 0 ]] || return $(touch_pkg_result $PKG post $RESULT "$MSG")

  if [[ $INSTANCE = first ]] ; then
    if ! docker login -u "$UNITY_DOCKER_USER" -p "$UNITY_DOCKER_PASSWORD" ; then
      MSG='docker login failed using UNITY_DOCKER_USER/UNITY_DOCKER_PASSWORD'
      return $(touch_pkg_result $PKG post 1 "$MSG")
    fi
  else
    docker-compose -f $YAML stop
    docker-compose -f $YAML pull
  fi

  docker-compose -f $YAML up -d

  return $(touch_pkg_result $PKG post $?)
}

#------------------------------------------------------------------------------
# execute
# -----------------------------------------------------------------------------

case $1 in
  pre)  epages-unity-pre  $2 $3 ; exit $? ;;
  post)  epages-unity-post  $2 $3 ; exit $? ;;
  *) exit 1 ;;
esac

#------------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
