#!/bin/bash

#=======================================================================
# install-additional-cartridges.sh
#=======================================================================

########################################################################
# usage
########################################################################

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this program"
  exit 1
fi

echo $BASH | grep -q '/bash$' && IS_BASH=1 || IS_BASH=
if [ -z "$BASH_VERSION" -o -z "$IS_BASH" ] ; then
  echo "$0: this script can only be executed by bash"
  exit 1
fi

LANG=C # use base language for standard output of some commands
LC_ALL=C

#------------------------------------------------------------------------------
# arguments
#------------------------------------------------------------------------------

usage() {
  [[ -n $1 ]] && echo -e "Error:\n  $@"
  echo "
Usage:
  /var/epages/install-additional-cartridges.sh -c CONFIGFILE

Usage (piped):
Options:
  -c: read configuration from CONFIGFILE, see example config in
      /srv/epages/etc/sysconfig/install-epages.conf.sample
"
  exit 2
}

unset CONFIGFILE
while getopts c: c ; do
  case $c in
    c) CONFIGFILE=$OPTARG ;;
    *) usage ;;
  esac
done

[[ -n $CONFIGFILE ]] || usage "Option missing: -c CONFIGFILE"
ls $CONFIGFILE >/dev/null || usage "Option -c with missing CONFIGFILE"

#-----------------------------------------------------------------------
# ECHO
#-----------------------------------------------------------------------

ECHO () {
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  echo -e "[${0##*/}:$LINE_NUMBER] $@"
}

########################################################################
# execution
########################################################################

PATH=$PATH:/var/epages:/var/epages/bin

. /etc/default/epages6

#-----------------------------------------------------------------------
# get new lines for epages.conf
#-----------------------------------------------------------------------

EPAGES_CONF_NEW=$(ini-config.sh -f "$CONFIGFILE" -s 'general definitions' -k epages-conf -G)
EPAGES_CONF_NEW=${EPAGES_CONF_NEW:+$(absolute_path "$EPAGES_CONF_NEW" "$CONFIGFILE")}

#-----------------------------------------------------------------------
# get new cartridges to install
#-----------------------------------------------------------------------

# key is storetype and value are cartridges to be installed
# e.g. NEW_CARTRIDGES[Store]='DE_EPAGES::ELogistics DE_EPAGES::Envialia'
declare -A NEW_CARTRIDGES

# for each key 'mysqld' in "$CONFIGFILE" get $MYSQLD_CONF
# for each $MYSQLD_CONF found get BU and its additional cartridges
while read key value ; do
  [[ $key != mysqld || -z $value ]] && continue
  MYSQLD_CONF=$(absolute_path "$value" "$CONFIGFILE")
  [[ -f $MYSQLD_CONF ]] || continue

  for section in $(ini-config.sh -f "$MYSQLD_CONF" -g) ; do
    storetype=$(ini-config.sh -f "$MYSQLD_CONF" -s $section -k StoreType -G)
    [[ -z $storetype ]] && continue
    for cartridge in $(ini-config.sh -f "$MYSQLD_CONF" -s $section -k StoreType.Add -G) ; do
      echo ${NEW_CARTRIDGES[$storetype]} | grep -q "\b$cartridge\b" && continue
      NEW_CARTRIDGES[$storetype]="${NEW_CARTRIDGES[$storetype]} $cartridge"
    done
  done
done < <(tr '=' ' ' < "$CONFIGFILE")

#-----------------------------------------------------------------------
# update epages.conf
#-----------------------------------------------------------------------

if [[ -f $EPAGES_CONF_NEW ]] ; then
  for cartridge in $(echo ${NEW_CARTRIDGES[@]} | tr ' ' '\n' | sort | uniq)
  do
    ini-config.sh -f "$EPAGES_CONF_NEW" -s "$cartridge" -G \
      | while IFS== read key value ; do 
      ECHO $LINENO "set_ini_value --overwrite $EPAGES_CONFIG/epages.conf $key $value $cartridge"
      set_ini_value --overwrite $EPAGES_CONFIG/epages.conf "$key" "$value" "$cartridge"
    done
  done
fi

#-----------------------------------------------------------------------
# for each business unit, add cartridges 
#-----------------------------------------------------------------------

for ST in "${!NEW_CARTRIDGES[@]}" ; do
  for BU in $(grep -lP "^\s*StoreType\s*=\s*$ST\s*$" $EPAGES_CONFIG/Database.d/*.conf) ; do
    BU=${BU##*/}
    ECHO $LINENO "add-cartridges-to-bu.sh -s ${BU%.conf} -a '${NEW_CARTRIDGES[$ST]}'"
    add-cartridges-to-bu.sh -s "${BU%.conf}" -a "${NEW_CARTRIDGES[$ST]}"
  done
done

#-----------------------------------------------------------------------
# exit
#-----------------------------------------------------------------------

exit 0
